"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
class WkhtmltopdfArguments {
    constructor(layoutPath, options) {
        this.layoutPath = layoutPath;
        this.options = options;
        this.args = [];
        this.parseGlobal();
        this.parseHeader();
        this.parseFooter();
        this.parseParts();
    }
    get arguments() {
        return this.args;
    }
    parseGlobal() {
        this.addArgument('--margin-bottom', this.options.marginBottom);
        this.addArgument('--margin-left', this.options.marginLeft);
        this.addArgument('--margin-right', this.options.marginRight);
        this.addArgument('--margin-top', this.options.marginTop);
        this.addArgument('--orientation', this.options.orientation);
        this.addArgument('--page-size', this.options.pageSize);
        this.addArgument('--page-height', this.options.pageHeight);
        this.addArgument('--page-width', this.options.pageWidth);
    }
    parseHeader() {
        this.parseHeaderFooter('header', this.options.header);
    }
    parseFooter() {
        this.parseHeaderFooter('footer', this.options.footer);
    }
    parseHeaderFooter(prefix, options) {
        if (options == null || options.html == null) {
            return;
        }
        this.addArgument(`--${prefix}-html`, path.join(this.layoutPath, options.html));
        this.addArgument(`--${prefix}-spacing`, options.spacing);
    }
    parseParts() {
        for (let key in this.options.parts) {
            const part = this.options.parts[key];
            if (part == null) {
                continue;
            }
            switch (part.type) {
                case 'cover':
                    this.parseCover(part);
                    break;
                case 'toc':
                    this.parseToc(part);
                    break;
                case 'content':
                    this.parseContent(part);
                    break;
                default:
                    throw new Error(`Unsupported part type '${part.type}' for part '${key}'!`);
                    break;
            }
        }
    }
    parseCover(part) {
        this.addArgument('cover', path.join(this.layoutPath, part.html));
    }
    parseToc(part) {
        this.args.push('toc');
        if (part.xslStyleSheet != null) {
            this.addArgument('--xsl-style-sheet', path.join(this.layoutPath, part.xslStyleSheet));
        }
    }
    parseContent(part) {
        this.args.push(path.join(this.layoutPath, part.html));
    }
    addArgument(name, value) {
        if (value === undefined || value === null) {
            return;
        }
        this.args.push(name);
        this.args.push(value);
    }
}
function wkhtmltopdfArguments(layoutPath, options) {
    return new WkhtmltopdfArguments(layoutPath, options).arguments;
}
exports.wkhtmltopdfArguments = wkhtmltopdfArguments;
function* allPaths(options) {
    if (options.header != null && options.header.html != null) {
        yield { path: options.header.html, data: options.header.data };
    }
    if (options.footer != null && options.footer.html != null) {
        yield { path: options.footer.html, data: options.footer.data };
    }
    for (let key in options.parts) {
        const part = options.parts[key];
        if (part.html != null) {
            yield { path: part.html, data: part.data };
        }
        if (part.type == 'toc' && part.xslStyleSheet != null) {
            yield { path: part.xslStyleSheet, data: part.data };
        }
    }
}
exports.allPaths = allPaths;
