"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const winston = require("winston");
const file_service_1 = require("./file-service");
const template_service_1 = require("./template-service");
const pdf_options_parser_1 = require("../helpers/pdf-options-parser");
class LayoutService {
    constructor() {
        this.layoutPath = path.join(__dirname, '../../layouts');
    }
    /**
     * Resolves the path of the given layout.
     * This function will try the following locations:
     * - layout path itself (if it is an absolute path)
     * - layout path relative to the documents location
     * - layout path inside the common layout directory
     *
     * @param layout The layout path or partial path to resolve
     * @param documentPath The path to the current document
     */
    resolveLayoutPathAsync(layout, documentPath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (path.isAbsolute(layout)) {
                if (yield file_service_1.default.existsAsync(layout)) {
                    return layout;
                }
                throw new Error(`Cannot find the layout '${layout}'.`);
            }
            const layoutInDocumentPath = path.join(path.dirname(documentPath), layout);
            if (yield file_service_1.default.existsAsync(layoutInDocumentPath)) {
                return layoutInDocumentPath;
            }
            const layoutInLayoutPath = path.join(this.layoutPath, layout);
            if (yield file_service_1.default.existsAsync(layoutInLayoutPath)) {
                return layoutInLayoutPath;
            }
            throw new Error(`Cannot find the layout '${layout}'.`);
        });
    }
    applyLayoutAsync(layoutPath, markdownAsHtml, options, additionalData) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = Object.assign({
                layoutPath: file_service_1.default.toAbsoluteFileUrl(layoutPath) + '/',
                markdown: markdownAsHtml,
                document: options.document,
                env: {
                    inputFile: path.basename(options.documentPath),
                    outputFile: path.basename(options.outputPath)
                }
            }, additionalData);
            const tempDirectory = yield this.getTempDirectoryAsync(options);
            const templateService = new template_service_1.TemplateService(options);
            for (let file of pdf_options_parser_1.allPaths(options.pdf)) {
                winston.debug('Applying template to', file);
                const template = yield file_service_1.default.readFileAsync(path.join(layoutPath, file.path));
                const targetFile = path.join(tempDirectory.path, path.basename(path.join(layoutPath, file.path)));
                const fileData = Object.assign({ layoutData: file.data }, data);
                const content = yield templateService.applyTemplateAsync(template.toString(), fileData);
                yield file_service_1.default.writeFileAsync(targetFile, content);
            }
            return tempDirectory;
        });
    }
    getTempDirectoryAsync(options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (options.tempPath == null) {
                return yield file_service_1.default.createTempDirectoryAsync();
            }
            else {
                yield file_service_1.default.createDirectoryRecursiveAsync(options.tempPath);
                return new file_service_1.TempPath(options.tempPath);
            }
        });
    }
}
exports.LayoutService = LayoutService;
exports.default = new LayoutService();
