"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const winston = require("winston");
const file_service_1 = require("../file-service");
/**
 * Includes markdown files.
 *
 * document.md:
 * # title
 * @(include file1.md)
 *
 * file1.md
 * ## subtitle
 *
 * will be merged to:
 * # title
 * ## subtitle
 */
class IncludePlugin {
    constructor(md) {
        this.md = md;
        this.dependencyManager = new DependencyManager();
    }
    parse(state, startLine, endLine, silent) {
        const beginPos = state.bMarks[startLine];
        const endPos = state.eMarks[startLine];
        const content = state.src.substring(beginPos, endPos);
        const match = IncludePlugin.REGEX.exec(content);
        if (!match) {
            return false;
        }
        // don't insert any tokens in silent mode
        if (silent) {
            return true;
        }
        state.line = startLine + 1;
        state.tokens.push({
            type: IncludePlugin.IDENTIFIER,
            lines: [startLine],
            level: state.level,
            meta: {
                match: match
            }
        });
        return true;
    }
    render(tokens, idx, options, env, renderer) {
        const token = tokens[idx];
        const match = token.meta.match;
        const includeFileName = match[1];
        const currentFile = this.currentFile(env);
        const currentBasePath = path.dirname(currentFile);
        const fullIncludeFileName = path.join(currentBasePath, includeFileName);
        winston.debug('Include markdown file %s relative to %s', includeFileName, currentBasePath);
        this.dependencyManager.with(currentFile);
        this.dependencyManager.check(fullIncludeFileName);
        const fileContent = file_service_1.default.readFile(fullIncludeFileName);
        const fileRendered = this.md.render(fileContent.toString(), { path: fullIncludeFileName });
        this.dependencyManager.end();
        return fileRendered;
    }
    currentFile(env) {
        if (!env.path) {
            throw new Error('No environment variable "path" provided.');
        }
        return env.path;
    }
    static register(md) {
        const plugin = new IncludePlugin(md);
        md.block.ruler.before('code', IncludePlugin.IDENTIFIER, plugin.parse.bind(plugin));
        md.renderer.rules[IncludePlugin.IDENTIFIER] = plugin.render.bind(plugin);
        return plugin;
    }
}
IncludePlugin.IDENTIFIER = 'include';
IncludePlugin.REGEX = /^\s*@\(include \s*([\w\/]+\.md)\s*\)\s*$/;
exports.IncludePlugin = IncludePlugin;
class DependencyManager {
    constructor() {
        this.files = [];
    }
    with(file) {
        this.files.push(file);
    }
    check(file) {
        if (this.files.indexOf(file) === -1) {
            return;
        }
        const path = this.files.join(' -> ') + ' -> ' + file;
        throw new Error(`There is a recursive loop in the path ${path}.`);
    }
    end() {
        this.files.splice(-1);
    }
}
