"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const winston = require("winston");
const file_service_1 = require("./file-service");
const layout_service_1 = require("./layout-service");
class OptionsService {
    constructor() {
        this.OPTIONS_FILE = 'options.json';
        this.OPTIONS_EXT = '.json';
    }
    // TODO: Is there a better naming for fallback & check
    consolidateAsync(options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fallbackOptionsToDocument(options);
            // Set fallback layout because this is needed for the layout fallback
            options.layout = yield this.getAndCheckLayoutExistanceAsync(options);
            winston.debug('Resolved layout', options.layout);
            yield this.fallbackOptionsToLayout(options);
            yield this.fallbackOptionsToDefault(options);
        });
    }
    fallbackOptionsToDocument(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const documentOptionsFile = file_service_1.default.changeExt(options.documentPath, this.OPTIONS_EXT);
            const documentOptions = yield this.loadOptionsByFile(documentOptionsFile);
            if (documentOptions != null) {
                winston.debug('Loading document options', documentOptionsFile);
                this.applyFallbackOptions(options, documentOptions);
                options.document = documentOptions.document || {};
            }
        });
    }
    getAndCheckLayoutExistanceAsync(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const layout = options.layout || 'document';
            return yield layout_service_1.default.resolveLayoutPathAsync(layout, options.documentPath);
        });
    }
    fallbackOptionsToLayout(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const layoutOptionsFile = path.join(options.layout, this.OPTIONS_FILE);
            const layoutOptions = yield this.loadOptionsByFile(layoutOptionsFile);
            if (layoutOptions != null) {
                winston.debug('Loading layout options', layoutOptionsFile);
                this.applyFallbackOptions(options, { pdf: layoutOptions });
            }
        });
    }
    fallbackOptionsToDefault(options) {
        options.outputPath = options.outputPath || file_service_1.default.changeExt(options.documentPath, '.pdf');
        this.applyFallbackOptions(options, {
            language: 'en',
            writeMetadata: true,
            pdf: {
                wkhtmltopdfPath: require('wkhtmltopdf-installer').path,
                pageSize: 'A4',
                orientation: 'Portrait'
            }
        });
    }
    loadOptionsByFile(file) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!(yield file_service_1.default.existsAsync(file))) {
                return null;
            }
            let content = yield file_service_1.default.readFileAsync(file);
            return JSON.parse(content.toString());
        });
    }
    applyFallbackOptions(options, fallback) {
        options.language = options.language || fallback.language;
        options.writeMetadata = options.writeMetadata == null ? fallback.writeMetadata : options.writeMetadata;
        options.pdf = options.pdf || {};
        options.pdf.header = options.pdf.header || {};
        options.pdf.footer = options.pdf.footer || {};
        options.pdf.parts = options.pdf.parts || {};
        if (fallback.pdf == null) {
            return;
        }
        options.pdf.wkhtmltopdfPath = options.pdf.wkhtmltopdfPath || fallback.pdf.wkhtmltopdfPath;
        options.pdf.marginBottom = options.pdf.marginBottom || fallback.pdf.marginBottom;
        options.pdf.marginLeft = options.pdf.marginLeft || fallback.pdf.marginLeft;
        options.pdf.marginRight = options.pdf.marginRight || fallback.pdf.marginRight;
        options.pdf.marginTop = options.pdf.marginTop || fallback.pdf.marginTop;
        options.pdf.orientation = options.pdf.orientation || fallback.pdf.orientation;
        options.pdf.pageSize = options.pdf.pageSize || fallback.pdf.pageSize;
        options.pdf.pageHeight = options.pdf.pageHeight || fallback.pdf.pageHeight;
        options.pdf.pageWidth = options.pdf.pageWidth || fallback.pdf.pageWidth;
        if (fallback.pdf.header != null) {
            options.pdf.header.html = options.pdf.header.html || fallback.pdf.header.html;
            options.pdf.header.spacing = options.pdf.header.spacing || fallback.pdf.header.spacing;
        }
        if (fallback.pdf.footer != null) {
            options.pdf.footer.html = options.pdf.footer.html || fallback.pdf.footer.html;
            options.pdf.footer.spacing = options.pdf.footer.spacing || fallback.pdf.footer.spacing;
        }
        if (fallback.pdf.parts == null) {
            return;
        }
        for (let key in fallback.pdf.parts) {
            if (fallback.pdf.parts[key] === false) {
                delete options.pdf.parts[key];
                continue;
            }
            if (options.pdf.parts[key] == null) {
                options.pdf.parts[key] = fallback.pdf.parts[key];
                continue;
            }
            options.pdf.parts[key].type = options.pdf.parts[key].type || fallback.pdf.parts[key].type;
            options.pdf.parts[key].html = options.pdf.parts[key].html || fallback.pdf.parts[key].html;
            switch (options.pdf.parts[key].type) {
                case 'toc':
                    const optionsToc = options.pdf.parts[key];
                    const fallbackToc = fallback.pdf.parts[key];
                    optionsToc.xslStyleSheet = optionsToc.xslStyleSheet || fallbackToc.xslStyleSheet;
                    break;
            }
        }
    }
}
exports.OptionsService = OptionsService;
exports.default = new OptionsService();
